# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2016-02-10.      #
#                                                           #
# Ruby Bindings Version 2.1.8                               #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

module Tinkerforge
  # Battery-backed real-time clock
  class BrickletRealTimeClock < Device
    DEVICE_IDENTIFIER = 268 # :nodoc:
    DEVICE_DISPLAY_NAME = 'Real-Time Clock Bricklet' # :nodoc:

    FUNCTION_SET_DATE_TIME = 1 # :nodoc:
    FUNCTION_GET_DATE_TIME = 2 # :nodoc:
    FUNCTION_GET_TIMESTAMP = 3 # :nodoc:
    FUNCTION_SET_OFFSET = 4 # :nodoc:
    FUNCTION_GET_OFFSET = 5 # :nodoc:
    FUNCTION_GET_IDENTITY = 255 # :nodoc:

    WEEKDAY_MONDAY = 1 # :nodoc:
    WEEKDAY_TUESDAY = 2 # :nodoc:
    WEEKDAY_WEDNESDAY = 3 # :nodoc:
    WEEKDAY_THURSDAY = 4 # :nodoc:
    WEEKDAY_FRIDAY = 5 # :nodoc:
    WEEKDAY_SATURDAY = 6 # :nodoc:
    WEEKDAY_SUNDAY = 7 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt> and adds it to
    # the IP Connection <tt>ipcon</tt>.
    def initialize(uid, ipcon)
      super uid, ipcon

      @api_version = [2, 0, 0]

      @response_expected[FUNCTION_SET_DATE_TIME] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_DATE_TIME] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_TIMESTAMP] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_SET_OFFSET] = RESPONSE_EXPECTED_FALSE
      @response_expected[FUNCTION_GET_OFFSET] = RESPONSE_EXPECTED_ALWAYS_TRUE
      @response_expected[FUNCTION_GET_IDENTITY] = RESPONSE_EXPECTED_ALWAYS_TRUE

    end

    # Sets the current date (including weekday) and the current time with hundredths
    # of a second resolution.
    # 
    # Possible value ranges:
    # 
    # * Year: 2000 to 2099
    # * Month: 1 to 12 (January to December)
    # * Day: 1 to 31
    # * Hour: 0 to 23
    # * Minute: 0 to 59
    # * Second: 0 to 59
    # * Centisecond: 0 to 99
    # * Weekday: 1 to 7 (Monday to Sunday)
    # 
    # If the backup battery is installed then the real-time clock keeps date and
    # time even if the Bricklet is not powered by a Brick.
    # 
    # The real-time clock handles leap year and inserts the 29th of February
    # accordingly. But leap seconds, time zones and daylight saving time are not
    # handled.
    def set_date_time(year, month, day, hour, minute, second, centisecond, weekday)
      send_request(FUNCTION_SET_DATE_TIME, [year, month, day, hour, minute, second, centisecond, weekday], 'S C C C C C C C', 0, '')
    end

    # Returns the current date (including weekday) and the current time of the
    # real-time clock with hundredths of a second resolution.
    def get_date_time
      send_request(FUNCTION_GET_DATE_TIME, [], '', 9, 'S C C C C C C C')
    end

    # Returns the current date and the time of the real-time clock converted to
    # milliseconds. The timestamp has an effective resolution of hundredths of a
    # second.
    def get_timestamp
      send_request(FUNCTION_GET_TIMESTAMP, [], '', 8, 'q')
    end

    # Sets the offset the real-time clock should compensate for in 2.17 ppm steps
    # between -277.76 ppm (-128) and +275.59 ppm (127).
    # 
    # The real-time clock time can deviate from the actual time due to the frequency
    # deviation of its 32.768 kHz crystal. Even without compensation (factory
    # default) the resulting time deviation should be at most ±20 ppm (±52.6
    # seconds per month).
    # 
    # This deviation can be calculated by comparing the same duration measured by the
    # real-time clock (``rtc_duration``) an accurate reference clock
    # (``ref_duration``).
    # 
    # For best results the configured offset should be set to 0 ppm first and then a
    # duration of at least 6 hours should be measured.
    # 
    # The new offset (``new_offset``) can be calculated from the currently configured
    # offset (``current_offset``) and the measured durations as follow::
    # 
    #   new_offset = current_offset - round(1000000 * (rtc_duration - ref_duration) / rtc_duration / 2.17)
    # 
    # If you want to calculate the offset, then we recommend using the calibration
    # dialog in Brick Viewer, instead of doing it manually.
    # 
    # The offset is saved in the EEPROM of the Bricklet and only needs to be
    # configured once.
    def set_offset(offset)
      send_request(FUNCTION_SET_OFFSET, [offset], 'c', 0, '')
    end

    # Returns the offset as set by BrickletRealTimeClock#set_offset.
    def get_offset
      send_request(FUNCTION_GET_OFFSET, [], '', 1, 'c')
    end

    # Returns the UID, the UID where the Bricklet is connected to, 
    # the position, the hardware and firmware version as well as the
    # device identifier.
    # 
    # The position can be 'a', 'b', 'c' or 'd'.
    # 
    # The device identifier numbers can be found :ref:`here <device_identifier>`.
    # |device_identifier_constant|
    def get_identity
      send_request(FUNCTION_GET_IDENTITY, [], '', 25, 'Z8 Z8 k C3 C3 S')
    end

  end
end
